@@ -6,11 +6,8 @@ get_string <- function(x, what = deparse(substitute(x))) {
     stop(sprintf("Expected a character vector for %s", what))
   }
 
-  ## TODO: this will get looked at in the next PR as we need to force
-  ## filenames better; it's not possible with this approach to error
-  ## if a file is missed through a typo.
-  if (length(x) == 1 && file.exists(x)) {
-    x <- paste(readLines(x), collapse = "\n")
+  if (refers_to_file(x)) {
+    x <- paste(readLines(x, warn = FALSE), collapse = "\n")
   } else if (length(x) > 1L) {
     x <- paste(x, collapse = "\n")
   }
@@ -19,6 +16,15 @@ get_string <- function(x, what = deparse(substitute(x))) {
 }
 
 
+refers_to_file <- function(x) {
+  ## good reasons not to be a file:
+  if (length(x) != 1 || inherits(x, "json") || grepl("{", x, fixed = TRUE)) {
+    return(FALSE)
+  }
+  file.exists(x)
+}
+
+
 `%||%` <- function(a, b) {
   if (is.null(a)) b else a
 }
@@ -70,3 +76,11 @@ note_imjv <- function(msg, is_interactive = interactive()) {
     message(msg)
   }
 }
+
+path_includes_dir <- function(x) {
+  !is.null(x) && basename(x) != x
+}
+
+is_absolute_path <- function(path) {
+  grepl("^(/|[A-Za-z]:)", path)
+}
